/*
 * XetPan: a libEtPan! based Mail User Agent
 * Copyright (C) 2002-2003 Melvin Hadasht
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef USE_GTKUI

#include <stdlib.h>
#include <errno.h>
#include <string.h>

#include <gtk/gtk.h>

#include "intl.h"

#include "logging.h"
#include "x_alloc.h"
#include "gtk-utils.h"
#include "mailbox.h"

#include "gtk-server-editor.h"

struct XePGtkServerEditor_s
{
  XePGtkServerType type;

  GtkWidget *widget; /* table */
  GtkWidget *host_entry;
  GtkWidget *port_spinbox;
  GtkWidget *auth_label;
  GtkWidget *auth_menu;
  GtkWidget *login_label;
  GtkWidget *login_entry;
  GtkWidget *passwd_label;
  GtkWidget *passwd_entry;
  GtkWidget *use_starttls;
  GtkWidget *use_ssl;
  GtkWidget *use_apop;
  GtkWidget *pop_menu;
  GtkWidget *pop_before_smtp_box;
  GtkWidget *pop_before_smtp;
  const char *pop_mbox;
  clist *mailboxes;
  XePServer *server;
};

enum
  {
    XEP_SEDITOR_AUTH        = 1 << 0,
    XEP_SEDITOR_STARTTLS    = 1 << 1,
    XEP_SEDITOR_SSL         = 1 << 2,
    XEP_SEDITOR_APOP        = 1 << 3,
    XEP_SEDITOR_POP_B4_SMTP = 1 << 4
  };

static GtkWidget *
GtkServerEditor_auth_menu_new(XePGtkServerEditor *seditor);

static void
GtkServerEditor_update_pop_menu(XePGtkServerEditor *seditor);

XePGtkServerEditor *
a_GtkServerEditor_new(void)
{
  XePGtkServerEditor *seditor;
  GtkWidget *label;
  GtkWidget *hbox;
  GtkTable *table;
  gint i;

  seditor = x_alloc(1, XePGtkServerEditor);
  
  if (seditor == NULL)
    {
      A_LOGGING_ERROR("malloc(): %s", strerror(errno));
      return NULL;
    }

  seditor->type = XEP_SERVEREDITOR_NONE;

  seditor->widget = gtk_table_new(7, 5, FALSE);
  table = GTK_TABLE(seditor->widget);
  gtk_table_set_row_spacings(table, 6);
  gtk_table_set_col_spacings(table, 12);

  label = a_GtkUtils_label_new_bold(_("Server:"));
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_widget_show(label);
  gtk_table_attach(table, label, 0, 5, 0, 1, GTK_FILL | GTK_EXPAND, 0, 0, 0);

  /* Host name and port */
  i = 1;
  seditor->host_entry = 
    a_GtkUtils_table_add_entry(table, _("_Host:"), NULL,
			       _("Host name of the server)"),
			       1, i);

  label = gtk_label_new_with_mnemonic(_("Po_rt:"));
  gtk_widget_show(label);
  gtk_table_attach(table, label, 3, 4, i, i + 1, 0, 0, 0, 0);

  seditor->port_spinbox = gtk_spin_button_new_with_range(0, 0xFFFF, 1);
  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(seditor->port_spinbox), 0);
  gtk_table_attach(table, seditor->port_spinbox, 4, 5, i, i+1, 
		   0, GTK_SHRINK, 0, 0);
  gtk_label_set_mnemonic_widget(GTK_LABEL(label), seditor->port_spinbox);
  gtk_widget_show(seditor->port_spinbox);

  /* Authentication */
  i++;			       
  seditor->auth_label = gtk_label_new_with_mnemonic(_("_Authentication:"));
  gtk_misc_set_alignment(GTK_MISC(seditor->auth_label), 0.0, 0.5);
  gtk_widget_show(seditor->auth_label);
  gtk_table_attach(table, seditor->auth_label, 1, 2, i, i + 1, 
		   GTK_FILL, GTK_SHRINK, 0, 0);

  seditor->auth_menu = GtkServerEditor_auth_menu_new(seditor);
  gtk_label_set_mnemonic_widget(GTK_LABEL(seditor->auth_label), 
				seditor->auth_menu);
  gtk_widget_show(seditor->auth_menu);
  gtk_table_attach(table, seditor->auth_menu, 2, 5, i, i + 1, 
		   GTK_FILL | GTK_EXPAND, 0, 0, 0);
  a_GtkUtils_set_tip(seditor->auth_menu, _("None: no authentication needed\n"
					   "Plain: traditional login/password\n"
					   "CRAM-MD5: \n"
					   "Login: proprietary login/password"));

  /* Login */
  i++;
  seditor->login_label = gtk_label_new_with_mnemonic(_("_Login:"));
  seditor->login_entry = a_GtkUtils_table_add_entry_full(table, NULL, 
							 seditor->login_label,
							 _("User's login"),
							 1, 3, i, 1);
  /* Password */
  i++;
  seditor->passwd_label = gtk_label_new_with_mnemonic(_("_Password:"));
  seditor->passwd_entry = a_GtkUtils_table_add_entry_full(table, NULL,
							 seditor->passwd_label,
							 _("User's password"),
							  1, 3, i, 1);

  /* Connection type */
  i++;
  hbox = gtk_hbox_new(TRUE, 12);
  gtk_widget_show(hbox);
  gtk_table_attach(table, hbox, 1, 5, i, i + 1, 
		   GTK_FILL | GTK_EXPAND, 0, 0, 0);

  /* STARTTLS */
  seditor->use_starttls = gtk_check_button_new_with_mnemonic(_("Use START_TLS"));
  gtk_widget_show(seditor->use_starttls);
  gtk_box_pack_start(GTK_BOX(hbox), seditor->use_starttls, TRUE, TRUE, 0);

  /* SSL */
  seditor->use_ssl = gtk_check_button_new_with_mnemonic(_("Use _SSL"));
  gtk_widget_show(seditor->use_ssl);
  gtk_box_pack_start(GTK_BOX(hbox), seditor->use_ssl, TRUE, TRUE, 0);

  /* APOP */
  seditor->use_apop = gtk_check_button_new_with_mnemonic(_("Use A_POP"));
  gtk_widget_show(seditor->use_apop);
  gtk_box_pack_start(GTK_BOX(hbox), seditor->use_apop, TRUE, TRUE, 0);

  /* POP before SMTP */
  i++;
  seditor->pop_before_smtp_box = hbox = gtk_hbox_new(FALSE, 12);

  seditor->pop_before_smtp = 
    gtk_check_button_new_with_mnemonic(_("POP before SMTP"));
  gtk_widget_show(seditor->pop_before_smtp);  
  gtk_box_pack_start(GTK_BOX(hbox), seditor->pop_before_smtp, FALSE, FALSE, 0);

  label = gtk_label_new_with_mnemonic(_("POP3 _Mailbox:"));
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
				      
  seditor->pop_menu = gtk_option_menu_new();
  gtk_widget_show(seditor->pop_menu);
  gtk_box_pack_start(GTK_BOX(hbox), seditor->pop_menu, TRUE, TRUE, 0);

  gtk_widget_show(hbox);
  
  gtk_table_attach(table, hbox, 1, 5, i, i + 1, 
		   GTK_FILL | GTK_EXPAND, 0, 0, 0);
  
  seditor->pop_mbox = NULL;
  seditor->mailboxes = NULL;
  seditor->server = NULL;

  gtk_widget_show(seditor->widget);
  return seditor;
}

void
a_GtkServerEditor_free(XePGtkServerEditor *seditor)
{
  gtk_widget_destroy(seditor->widget);
  x_free(seditor);
}

GtkWidget *
a_GtkServerEditor_get_widget(XePGtkServerEditor *seditor)
{
  return seditor->widget;
}

void
a_GtkServerEditor_set_mailboxes(XePGtkServerEditor *seditor, clist *mailboxes)
{
  seditor->mailboxes = mailboxes;

  GtkServerEditor_update_pop_menu(seditor);
}

static guint
GtkServerEditor_get_widgets_from_type(XePGtkServerType type)
{
  switch (type)
    {
    case XEP_SERVEREDITOR_IMAP:
      return XEP_SEDITOR_AUTH | XEP_SEDITOR_STARTTLS | XEP_SEDITOR_SSL;
    case XEP_SERVEREDITOR_NNTP:
      return 0;
    case XEP_SERVEREDITOR_POP3:
      return XEP_SEDITOR_AUTH | XEP_SEDITOR_STARTTLS | XEP_SEDITOR_SSL |
	XEP_SEDITOR_APOP | XEP_SEDITOR_POP_B4_SMTP;
    case XEP_SERVEREDITOR_SMTP:
      return XEP_SEDITOR_AUTH | XEP_SEDITOR_STARTTLS | XEP_SEDITOR_SSL;
    default:
      return 0;
    }
}

void
a_GtkServerEditor_set_type(XePGtkServerEditor *seditor, XePGtkServerType type)
{
  guint widgets;
  widgets = GtkServerEditor_get_widgets_from_type(type);

  seditor->type = type;

  if (widgets & XEP_SEDITOR_AUTH)
    {
#if 0
      GtkServerEditor_set_auth(seditor, server->auth);
#endif
      gtk_widget_show(seditor->auth_label);
      gtk_widget_show(seditor->auth_menu);
    }
  else
    {
      gtk_widget_hide(seditor->auth_label);
      gtk_widget_hide(seditor->auth_menu);
    }

  if (widgets & XEP_SEDITOR_STARTTLS)
    {
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(seditor->use_starttls),
				   a_Server_get_starttls(seditor->server));
      gtk_widget_show(seditor->use_starttls);
      
    }
  else
    {
      gtk_widget_show(seditor->use_starttls);
    }

  if (widgets & XEP_SEDITOR_SSL)
    {
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(seditor->use_starttls),
				   a_Server_get_ssl(seditor->server));
      gtk_widget_show(seditor->use_ssl);
    }
  else
    {
      gtk_widget_hide(seditor->use_ssl);
    }

  if (widgets & XEP_SEDITOR_APOP)
    {
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(seditor->use_starttls),
				   a_Server_get_apop(seditor->server));
      gtk_widget_show(seditor->use_apop);
    }
  else
    {
      gtk_widget_hide(seditor->use_apop);
    }
  if (widgets & XEP_SEDITOR_POP_B4_SMTP)
    {
      /* FIXME: implement */
      gtk_widget_show(seditor->pop_before_smtp_box);
    }
  else
    {
      gtk_widget_hide(seditor->pop_before_smtp_box);
    }
  a_GtkUtils_entry_set_text(GTK_ENTRY(seditor->host_entry),
			    a_Server_get_host(seditor->server));

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(seditor->port_spinbox),
			    a_Server_get_port(seditor->server));
  a_GtkUtils_entry_set_text(GTK_ENTRY(seditor->login_entry),
			    a_Server_get_login(seditor->server));

  a_GtkUtils_entry_set_text(GTK_ENTRY(seditor->passwd_entry),
			    a_Server_get_password(seditor->server));
}

static GtkWidget *
GtkServerEditor_auth_menu_new(XePGtkServerEditor *seditor)
{
  GtkItemFactory *item_factory;
  GtkItemFactoryEntry entries[] = {
    {_("/Plain"), NULL, NULL, 0, NULL},
    {_("/None"), NULL, NULL, 0, NULL},
    {_("/CRAM-MD5"), NULL, NULL, 0, NULL},
    {_("/Login"), NULL, NULL, 0, NULL}
  };

  item_factory = gtk_item_factory_new(GTK_TYPE_OPTION_MENU, "<Server>/", NULL);
  gtk_item_factory_create_items(item_factory,
				sizeof(entries)/sizeof(entries[0]),
				entries, seditor);
  return gtk_item_factory_get_widget(item_factory, "<Server>/");
}

static void
GtkServerEditor_update_pop_menu(XePGtkServerEditor *seditor)
{
  GtkWidget *menu;
  GtkWidget *menuitem;
  clistiter *iter;
  XePMailbox *mailbox;
  guint selected = 0;
  guint i;
  const char *mbox_id;

  menu = gtk_menu_new();

  for (iter = clist_begin(seditor->mailboxes), i = 0;
       iter;
       iter = iter->next, i++)
    {
      mailbox = (XePMailbox *) iter->data;
      if (mailbox->type == XEP_MAILBOX_POP3)
	{
	  mbox_id = a_Mailbox_get_id(mailbox);
	  if (mbox_id && seditor->pop_mbox && 
	      strcmp(mbox_id, seditor->pop_mbox) == 0)
	    selected = i;
	  menuitem = gtk_menu_item_new_with_label(a_Mailbox_get_id(mailbox));
	  gtk_menu_shell_append(GTK_MENU_SHELL(seditor->pop_menu), menuitem);
	}
    }

  gtk_option_menu_remove_menu(GTK_OPTION_MENU(seditor->pop_menu));
  gtk_option_menu_set_menu(GTK_OPTION_MENU(seditor->pop_menu), menu);
  gtk_option_menu_set_history(GTK_OPTION_MENU(seditor->pop_menu), selected);

}

void
a_GtkServerEditor_set_server(XePGtkServerEditor *seditor, XePServer *server)
{
  seditor->server = server;
}

void
a_GtkServerEditor_apply_changes(XePGtkServerEditor *seditor)
{
  guint widgets;
  widgets = GtkServerEditor_get_widgets_from_type(seditor->type);

  if (widgets & XEP_SEDITOR_AUTH)
    {
#if 0
      GtkServerEditor_set_auth(seditor, server->auth);
#endif
    }

  if (widgets & XEP_SEDITOR_STARTTLS)
    {
      a_Server_set_starttls(seditor->server,
			    gtk_toggle_button_get_active
			    (GTK_TOGGLE_BUTTON(seditor->use_starttls)));
    }

  if (widgets & XEP_SEDITOR_SSL)
    {
      a_Server_set_ssl(seditor->server,
		       gtk_toggle_button_get_active
		       (GTK_TOGGLE_BUTTON(seditor->use_ssl)));      
    }

  if (widgets & XEP_SEDITOR_APOP)
    {
      a_Server_set_apop(seditor->server,
			gtk_toggle_button_get_active
			(GTK_TOGGLE_BUTTON(seditor->use_apop)));      
    }
  if (widgets & XEP_SEDITOR_POP_B4_SMTP)
    {
      A_LOGGING_ERROR("POP before SMPT Not implemented yet");
    }
  a_Server_set_host(seditor->server,
		    gtk_entry_get_text(GTK_ENTRY(seditor->host_entry)));
  a_Server_set_port(seditor->server,
		    gtk_spin_button_get_value
		    (GTK_SPIN_BUTTON(seditor->port_spinbox)));
  a_Server_set_login(seditor->server,
		     gtk_entry_get_text(GTK_ENTRY(seditor->login_entry)));

  a_Server_set_password(seditor->server,
			gtk_entry_get_text(GTK_ENTRY(seditor->passwd_entry)));
}

#endif
