/*
 * XetPan: a libEtPan! based Mail User Agent
 * Copyright (C) 2002 Melvin Hadasht
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef GTKUI_H
#define GTKUI_H

#include <sys/types.h> /* size_t used by regex */
#include <regex.h>     /* regcomp, regexec, regfree */

#include <gtk/gtkwidget.h>
#include <gtk/gtktreestore.h>

#include "xetpan.h"
#include "ui-types.h"
#include "manager.h"
#include "message.h"

#define XEP_STOCK_ICON              "xetpan-icon"
#define XEP_STOCK_STATUS            "xetpan-status"
#define XEP_STOCK_FLAG              "xetpan-flag"
#define XEP_STOCK_REPLY             "xetpan-reply"
#define XEP_STOCK_FORWARD           "xetpan-forward"
#define XEP_STOCK_REPLIED           "xetpan-replied"
#define XEP_STOCK_FORWARDED         "xetpan-forwarded"
#define XEP_STOCK_REPLIED_FORWARDED "xetpan-replied-forwarded"
#define XEP_STOCK_UNSEEN            GTK_STOCK_NO
#define XEP_STOCK_NEW               GTK_STOCK_YES
#define XEP_STOCK_DELETED            GTK_STOCK_DELETE

typedef struct XePGTKSummaryView_s XePGTKSummaryView;

struct XePGTKSummaryView_s
{
  GtkWidget *widget;
  GtkWidget *tree;
  GtkWidget *search_box;
  GtkWidget *search_icon;
  GtkWidget *search_entry;
  GtkWidget *stat_label;
  regex_t preg;
  gboolean valid_preg;
  XePFolderRef *folder_ref;
  XePUIMain *uimain;
};

#define a_GtkUI_foldersview_show(fview)   gtk_widget_show(fview->widget)
#define a_GtkUI_foldersview_widget(fview) GTK_WIDGET(fview->widget)
#define a_GtkUI_summaryview_show(sview)   gtk_widget_show(sview->widget)
#define a_GtkUI_summaryview_widget(sview) GTK_WIDGET(sview->widget)

void
a_GtkUI_init(XetPan *xetpan, int *argc, char ***argv);

int
a_GtkUI_init_check(XetPan *xetpan, int *argc, char ***argv);

void
a_GtkUI_main(XetPan *xetpan);

void
a_GtkUI_main_quit(XetPan *xetpan);

int
a_GtkUI_events_pending(XetPan *xetpan);

void
a_GtkUI_main_iteration(XetPan *xetpan);

void
a_GtkUI_first_launch(XetPan *xetpan);

void
a_GtkUI_compose_pending_requests(XetPan *xetpan);

XePUIMain*
a_GtkUI_mainview_create(XetPan *xetpan, char *terminal);

/*! Closes a main view UI and frees it resources.
 *  \param uimain the UI to close
 */
void
a_GtkUI_mainview_close(XePUIMain *uimain);

void
a_GtkUI_mainview_reply(XePUIMain *uimain);

void
a_GtkUI_mainview_forward(XePUIMain *uimain);

void
a_GtkUI_mainview_redirect(XePUIMain *uimain);

/*! Called whenever the foldersview list is updated.
 *  This function is called by a_UI_foldersview_list_update()
 *  It should update its view of the foldersview list
 *  \param fview the folders view 
 */
void
a_GtkUI_foldersview_list_updated(XePUIFoldersView *fview);

void
a_GtkUI_foldersview_list_refresh(XePUIFoldersView *fview);

void
a_GtkUI_summaryview_set_view(XePGTKSummaryView *sview,
			     XePFolderRef *folder_ref);

/*! Update and view the summary of a certain folder 
 *  \param fview the corresponding folder view
 *  \param folder_ref a pointer to the folder
 */
void
a_GtkUI_summaryview_set_view_async(XePGTKSummaryView *sview,
				   XePFolderRef *folder_ref);

void
a_GtkUI_summaryview_selection_delete(XePUIMain *uimain, XePMessageFlagsOp flag_op);

void
a_GtkUI_summaryview_selection_flag(XePUIMain *uimain, XePMessageFlagsOp flag_op);

void
a_GtkUI_summaryview_selection_seen(XePUIMain *uimain, XePMessageFlagsOp flag_op);

void
a_GtkUI_summaryview_find_match(XePGTKSummaryView *sview, 
			       gboolean forward);

void
a_GtkUI_summaryview_expunge(XePUIMain *uimain);

void
a_GtkUI_folder_updated(XePUIMain *uimain, XePFolder *folder);

void 
a_GtkUI_folder_refresh(XePUIMain *uimain, XePFolder *folder);

void
a_GtkUI_messageview_finalize(XePUIMessageView *mview, XePMessage *message);

void
a_GtkUI_messageview_mime_view_show(XePUIMessageView *mview,
				   XePMessage *message);

void
a_GtkUI_messageview_mime_view_block(XePUIMessageView *mview,
				    XePMessage *message);

void
a_GtkUI_messageview_mime_view_unblock(XePUIMessageView *mview,
				      XePMessage *message);

void
a_GtkUI_messageview_mime_show_selected(XePUIMessageView *mview,
				       XePMessage *message);

XePUIComposeWindow *
a_GtkUI_composewindow_create(XetPan *xetpan, char *terminal);

void
a_GtkUI_composewindow_set_message(XePUIComposeWindow *uicompose, 
				  XePMessage *message);

void
a_GtkUI_composewindow_set_body_text(XePUIComposeWindow *uicompose,
				    const char *text,
				    size_t text_size,
				    const char *text_charset);

void
a_GtkUI_composewindow_add_header(XePUIComposeWindow *uicompose, 
				 XePMessageHeaderType header_type,
				 const char *header_name,
				 const char *header_value);

void
a_GtkUI_composewindow_close(XePUIComposeWindow *uicompose);

XePUIMessageView *
a_GtkUI_messageview_create_standalone(XetPan *xetpan, 
				      char *terminal);
void
a_GtkUI_messageview_close(XePUIMessageView *mview);

/*! Display information about XetPan Email Client */
void
a_GtkUI_about(XePUIMain *uimain);

void
a_GtkUI_notify_error(XePUIMain *uimain);

void
a_GtkUI_notify_error_reset(XePUIMain *uimain);

/*! Function called when a message was modified (flags...) 
 *  When a message is modified (for example its flags),
 *  this function is called for every Gtk UI and is responsible
 *  for updating the summary view accordingly
 *  \param uimain
 *  \param message the message that was modified
 */
void
a_GtkUI_message_updated(XePUIMain *uimain, XePMessage *message);

int
a_GtkUI_create_io_watch(XePUI *ui, XePUIIOWatcher *watcher);
		
void
a_GtkUI_delete_io_watch(XePUI *ui, XePUIIOWatcher *watcher);

#endif
