/*
 * XetPan: a libEtPan! based Mail User Agent
 * Copyright (C) 2002-2003 Melvin Hadasht
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifndef MTA_H
#define MTA_H

#include <libetpan/clist.h>

#include "server.h"
#include "message-types.h"
#include "manager.h"

typedef enum
  {
    XEP_MTA_INVALID,
    XEP_MTA_COMMAND,
    XEP_MTA_SMTP
  } XePMTAType;

typedef struct XePMTA_s XePMTA;

struct XePMTA_s
{
  char *id;
  XePMTAType type;
  char *command;
  XePServer *server;
  XePMessage *message;
  int result; /**< Place holder for the result of SMTP send operation */
  XePManagerThread thread; 
};

clist *
a_MTA_list_new(void);

void
a_MTA_list_free(clist *mtas);

clist *
a_MTA_list_get(void);

XePMTA *
a_MTA_new(void);

void
a_MTA_free(XePMTA *mta);

void
a_MTA_set_id(XePMTA *mta, const char *id);

const char *
a_MTA_get_id(XePMTA *mta);

void
a_MTA_set_type(XePMTA *mta, XePMTAType type);

XePMTAType
a_MTA_get_type(XePMTA *mta);

void
a_MTA_set_command(XePMTA *mta, const char *command);

const char *
a_MTA_get_command(XePMTA *mta);

void
a_MTA_set_server(XePMTA *mta, XePServer *server);

XePServer *
a_MTA_get_server(XePMTA *mta);

/*! Send a message using a Mail Transfer Agent 
 *  \param mta the XePMTA structure describing the Mail Transfer Agent to use
 *  \param message the XePMessage structure containing the message to send.
 *         message must exist until sent_cb is called.
 *  \param sent_cb function to be called when processing the message has
 *         finished. Its first argument is the user_data parameter, the
 *         second argument is the result code passed as a pointer
 *         (a cast from (void *) to (int) must be done).
 *  \param user_data user data to pass as first argument to the send_cb
 */
void
a_MTA_send_message(XePMTA *mta, XePMessage *message, 
		   XePManagerMsgCb sent_cb, void *user_data,
		   const char *sent_folder_ref);

/*! Send a list of messages using a Mail Transfer Agent 
 *  \param mta the XePMTA structure describing the Mail Transfer Agent to use
 *  \param message_list a clist of XePMessage structures containing the messages
 *         to send. The list and its content must exist until sent_cb is called.
 *  \param sent_cb function to be called when processing the message has
 *         finished. Its first argument is the user_data parameter, the
 *         second argument is the result code passed as a pointer
 *         (a cast from (void *) to (int) must be done).
 *  \param user_data user data to pass as first argument to the send_cb
 */
void
a_MTA_send_message_list(XePMTA *mta, clist *message_list, 
			XePManagerMsgCb sent_cb, void *user_data);

/*! Send a message using an SMTP MTA
 *  This function is used from a thread, and is not meant to be used
 *  directly.
 *  \param mta the XePMTA structure describing the MTA. The XePMTA::message
 *         member must point to the message to be sent.
 *  \return 0 on success, -1 on failure.
 */
int
a_MTA_smtp_send_message(XePMTA *mta);

/*! Finds the MTA by its reference (id)
 *  \param mtas clist of registered MTAs
 *  \param ref the reference of the MTA. If NULL, the first MTA is returned
 *  \return the associated MTA. If ref is NULL, the first registered MTA is
 *          returned. Returns NULL if there is no MTAs, of if mtas is NULL
 */
XePMTA *
a_MTA_lookup_by_ref(const char *ref);

#endif
